/* soundmosaic, Copyright (c) 2001-2007 Steven Hazel <sah@thalassocracy.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 */
#ifndef __IO_BUFFER_H__
#define __IO_BUFFER_H__

#include <stdlib.h>
#include <stdio.h>

#ifndef _WIN32
  #define SOCKET int
#else
  #ifndef _SSIZE_T_DEFINED
    #ifdef  _WIN64
      typedef signed __int64    ssize_t;
    #else
      typedef _W64 signed int   ssize_t;
    #endif
    #define _SSIZE_T_DEFINED
  #endif
#endif


#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  unsigned char *data;
  size_t size;
  size_t read_pos;
  size_t write_pos;
  int failed;
} io_buffer;

void io_buffer_init(io_buffer *self, size_t size);
void io_buffer_reset(io_buffer *self);
void io_buffer_destroy(io_buffer *self);
void io_buffer_grow(io_buffer *self, size_t size);

void io_buffer_compact(io_buffer *self);
void io_buffer_discard_bytes(io_buffer *self, size_t num);

int io_buffer_is_empty(const io_buffer *self);
int io_buffer_is_full(const io_buffer *self);
size_t io_buffer_bytes_free(const io_buffer *self);
size_t io_buffer_bytes_used(const io_buffer *self);

int io_buffer_failed(const io_buffer *self);

ssize_t io_buffer_recv_all(io_buffer *self, SOCKET fd);
ssize_t io_buffer_send_all(io_buffer *self, SOCKET fd);

size_t io_buffer_fread_all(io_buffer *self, FILE *stream);
size_t io_buffer_fread(io_buffer *self, FILE *stream, size_t len);
size_t io_buffer_fwrite(io_buffer *self, FILE *stream, size_t len);

int io_buffer_peek_int(io_buffer *self, size_t len, size_t offset);

void io_buffer_read(io_buffer *self, char *dest, size_t len);
void io_buffer_write(io_buffer *self, const char *src, size_t len);

#ifdef __cplusplus
}
#endif

#endif /* __IO_BUFFER_H__ */
